<?php

namespace Drupal\config_export_import;

use Drupal\Component\Serialization\Exception\InvalidDataTypeException;
use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Config\FileStorage;

class ConfigExportImportManager
{
    public function getConfigContains($filter){
        $configs = \Drupal::configFactory()->listAll();
        $result = [];
        foreach ($configs as $config_name){
            if (is_string($filter) && strpos($config_name, $filter) !== false) {
                $result[] = $config_name ;
            }
            if (is_array($filter) && in_array($config_name, $filter) !== false) {
                $result[] = $config_name ;
            }
        }
        return $result ;
    }

    public function exportConfig($config_name,$path){
        $config = \Drupal::config($config_name) ;
        $data = $config->getOriginal();
        $config_settings = \Drupal::config("config_export_import.settings") ;
        $force = $config_settings->get('export');
        try {
            $file_path = DRUPAL_ROOT.$path.'/'.$config_name.'.yml' ;
            if (file_exists($file_path)) {
                if(!$force){
                    $output = Yaml::encode($data);
                    $status = $this->generateFileForce(DRUPAL_ROOT.$path,$config_name.'.yml',$output);
                    if($status){
                        \Drupal::messenger()->addMessage('exported Config '.$config_name);
                    }
                }else{
                    \Drupal::messenger()->addError('Config '.$config_name. ' exist already in folder');
                }
            }else{
                $output = Yaml::encode($data);
                $status = $this->generateFileForce(DRUPAL_ROOT.$path,$config_name.'.yml',$output);
                if($status){
                    \Drupal::messenger()->addMessage('exported Config '.$config_name);
                }
            }
        }
        catch (InvalidDataTypeException $e) {
            \Drupal::messenger()->addError($this->t('Invalid data detected for @name : %error', array('@name' => $config_name, '%error' => $e->getMessage())));
            return;
        }

    }
    public function generateFileForce($directory, $filename, $content)
    {
        $fileSystem = \Drupal::service('file_system');
        if (!is_dir($directory)) {
            if ($fileSystem->mkdir($directory, 0777, TRUE) === FALSE) {
                drupal_set_message(t('Failed to create directory ' . $directory), 'error');
                return FALSE;
            }
        }else{
            @chmod($directory  , 0777);
        }

        if (file_put_contents($directory . '/' . $filename , $content) === FALSE) {
            drupal_set_message(t('Failed to write file ' . $filename), 'error');
            return FALSE;
        }
        return TRUE;
    }
    public function importConfig($config_name,$path){
        $config_settings = \Drupal::config("config_export_import.settings") ;
        $force = $config_settings->get('import');
        $source = new FileStorage(DRUPAL_ROOT.$path);
        $config_storage = \Drupal::service('config.storage');

        if($source->exists($config_name)){

            if($config_storage->exists($config_name)){
                if(!$force){
                    $status =  $config_storage->write($config_name, $source->read($config_name));
                    if($status){
                        \Drupal::messenger()->addMessage('updated config '.$config_name);
                    }
                }else{
                    \Drupal::messenger()->addError('Config '.$config_name. ' exist already');
                }
            }else{
                $status = \Drupal::configFactory()->getEditable($config_name)
                    ->setData($source->read($config_name))
                    ->save();
                if($status){
                    \Drupal::messenger()->addMessage('Create Config '.$config_name);
                }
            }
        }else{
            \Drupal::messenger()->addError('File yml '.$config_name.' not exit in folder '.$path);
            return false;
        }
    }
    public  function readDirectory($directory,$format = 'json')
    {
        $path_file = [];
        if (is_dir($directory)) {
            $it = scandir($directory);
            if (!empty($it)) {
                foreach ($it as $fileinfo) {
                    $element =  $directory . "/" . $fileinfo;
                    if (is_dir($element) && substr($fileinfo, 0, strlen('.')) !== '.') {
                        $childs = $this->readDirectory($element,$format);
                        $path_file = array_merge($childs , $path_file);
                    }else{
                        if ($fileinfo && strpos($fileinfo, '.'.$format) !== FALSE) {
                            if (file_exists($element)) {
                                $path_file[] =  $directory . "/" . $fileinfo;
                            }
                        }
                    }
                }
            }
        }else{
            drupal_set_message(t('No permission to read directory ' . $directory), 'error');
            @chmod($directory  , 0777);
        }
        return $path_file;
    }

}